<?php

class d_cache
{
  public function __construct()
  {
    $this->dir = dirname(dirname(__FILE__)).'/cache/';
  }

  public static function get()
  {
    return new d_cache();
  }

  private $dir;

  private function file($index = false, $url = '')
  {
    return $this->dir.($index ? '0_index_' : '').md5(empty($url) ? $_SERVER['REQUEST_URI'] : $url);
  }


  public function save($buffer)
  {
    if($_SERVER['REQUEST_METHOD'] !== 'GET') return $buffer;

    if(empty($buffer) || !strstr(strtolower($buffer), '</body>') || !file_exists($this->dir) || !is_writable($this->dir)) return $buffer;

    if(!d_need_to_cache_this_url()) return $buffer;

    $index = d_is_index();

    $file = $this->file($index);
    if($handle = fopen($file, 'a+'))
    {
      if(flock($handle, LOCK_EX))
      {
        $content = gzencode($buffer, 9);

        ftruncate($handle, 0);
        rewind($handle);
        fwrite($handle, $content, strlen($content));
        fflush($handle);
        flock($handle, LOCK_UN);
      }
      fclose($handle);
    }

    return $buffer;
  }


  public function load()
  {
    if($_SERVER['REQUEST_METHOD'] !== 'GET' || d_is_robots_txt() || d_is_wp_login_page() || d_is_preview() || d_is_customize_preview() || d_is_wp_json() || d_is_admin_ajax() || d_is_ajax_php() || d_is_favicon()) return;

    if(d_need_to_cache_ignore_this_url()) return;

    $index = false;
    $file = $this->file();
    if(!file_exists($file))
    {
      $file = $this->file(true);
      $index = true;
    }

    if(!file_exists($file)) return;


    if(!$index && CACHE_LIFETIME > 0 && time() - filemtime($file) > CACHE_LIFETIME * 60) return;

    if($index && CACHE_INDEX_LIFETIME > 0 && time() - filemtime($file) > CACHE_INDEX_LIFETIME * 60) return;


    if($handle = fopen($file, 'a+'))
    {
      if(flock($handle, LOCK_EX))
      {
        clearstatcache();
        $filesize = filesize($file);
        if($filesize == 0) return;
        $buffer = fread($handle, $filesize);

        flock($handle, LOCK_UN);
      }
      else
      {
        return;
      }
      fclose($handle);
    }
    else
    {
      return;
    }


    $need_to_repack = (EXEC_PHP && EXEC_PHP_DO_NOT_CACHE) || SHOW_DEBUG_BAR;

    if($need_to_repack) $buffer = gzinflate(substr($buffer, 10, -8));

    if(EXEC_PHP && EXEC_PHP_DO_NOT_CACHE) $buffer = php_exec_all($buffer);

    if(SHOW_DEBUG_BAR) $buffer = d_show_debug_bar($buffer, true);

    if($need_to_repack) $buffer = gzencode($buffer, 5);


    $this->set_headers();

    echo $buffer;

    exit;
  }


  public function set_headers()
  {
    header('Content-Encoding: gzip', true);
    header('Content-Type: text/html; charset=utf-8', true); // get_option('blog_charset')
    header('Cache-Control: no-cache, must-revalidate, max-age=0', true);
    header('Expires: Wed, 11 Jan 1984 05:00:00 GMT', true);
    header('Pragma: no-cache', true);
  }



  public function check()
  {
    $status = '';

    if(!file_exists($this->dir)) mkdir($this->dir);

    if(!file_exists($this->dir))
    {
      $status .= 'Не найдена директория кэша <br><b style="color:#777; font-size:11px; font-weight:normal;">['.$this->dir.']</b><br><br>';
    }
    else
    {
      if(file_put_contents($this->dir.'/tmp.txt', '1') === false)
      {
        $status .= 'Директория кэша не доступна для записи <br><b style="color:#777; font-size:11px; font-weight:normal;">['.$this->dir.']</b><br>Установите права 0777 на папку с плагином.<br><br>';
      }
      else
      {
        unlink($this->dir.'/tmp.txt');
      }
    }

    return $status;
  }


  public function clear_posts($post_id = null)
  {
    if($post_id != null)
    {
      $file = $this->file(false, str_replace(home_url(), '', get_permalink($post_id)));
      if(file_exists($file)) unlink($file);
      return;
    }

    foreach(d_get_files($this->dir) as $file)
    {
      if(substr($file, 0, 7) != '0_index') unlink($this->dir.$file);
    }
  }

  public function clear_index()
  {
    foreach(d_get_files($this->dir) as $file)
    {
      if(substr($file, 0, 7) == '0_index') unlink($this->dir.$file);
    }
  }

  public function clear_db()
  {
    d_config::get()->remove_consts_regexp('/define\(\'DBCACHE_.*?\n/s');
  }

  public function clear_all()
  {
    $this->clear_posts();
    $this->clear_index();
    $this->clear_db();
  }


  public function count_posts()
  {
    $count = 0;
    foreach(d_get_files($this->dir) as $file) if(substr($file, 0, 7) != '0_index') $count++;
    return $count;
  }

  public function count_index()
  {
    $count = 0;
    foreach(d_get_files($this->dir) as $file) if(substr($file, 0, 7) == '0_index') $count++;
    return $count;
  }

  public function count_db()
  {
    return d_config::get()->count_consts_regexp('/DBCACHE/s');
  }


  public function action_save_or_delete_posts($id/*, $post, $update*/)
  {
    $post = get_post($id);

    if($post->post_status != 'publish') return;

    //if(wp_is_post_revision($post_id)) return;

    if(CACHE_CLEAR_ALL_WHEN_SAVE_OR_DELETE_POSTS) $this->clear_posts();
    else $this->clear_posts($id);

    if(CACHE_INDEX_CLEAR_ALL_WHEN_SAVE_OR_DELETE_POSTS) $this->clear_index();
  }


  public function action_comment_post($comment_id, $approved, $comment)
  {
    if($approved == true) $this->clear_posts($comment['comment_post_ID']);
  }

  public function action_comment_edit($comment_id, $comment)
  {
    $this->clear_posts($comment['comment_post_ID']);
  }

  public function action_transition_comment_status($new_status, $old_status, $comment)
  {
    $this->clear_posts($comment->comment_post_ID);
  }
}
