<?php

require_once('d_helpers.php');

define('DOMAIN_ID', str_replace('www.', '', $_SERVER['SERVER_NAME']));

class d_config
{
  public function __construct()
  {
    $this->init();

    $this->file = dirname(dirname(__FILE__)).'/d_config.php';
    $this->file_backup = dirname(dirname(__FILE__)).'/d_config.backup';
  }

  public static function get()
  {
    return new d_config();
  }

  private $file;
  private $file_backup;

  private $consts = array();

  private $categories = array();


  public function add($category)
  {
    $this->categories[] = $category;
    return $this;
  }

  public function load()
  {
    if(!file_exists($this->file))
    {
      file_put_contents($this->file, "<?php\n");
    }

    if($handle = fopen($this->file, 'a+'))
    {
      if(flock($handle, LOCK_EX))
      {
        clearstatcache();
        $filesize = filesize($this->file);
        $content = $filesize > 0 ? fread($handle, $filesize) : '';

        if(!$this->check_config($content)) $this->restore($this->file_backup, $handle);

        require_once($this->file);

        flock($handle, LOCK_UN);
      }
      fclose($handle);
    }

    foreach($this->consts as $const) if(!defined($const)) define($const, false);

    if(DISABLE_WP_VERIFY_NONCE) include_once('d_override_nonce.php');
  }



  public function save($name, $value)
  {
    if($handle = fopen($this->file, 'a+'))
    {
      if(flock($handle, LOCK_EX))
      {
        clearstatcache();
        $filesize = filesize($this->file);
        $content = $filesize > 0 ? fread($handle, $filesize) : '';

        $this->save_const($content, $name, $value);

        ftruncate($handle, 0);
        rewind($handle);
        fwrite($handle, $content, strlen($content));
        fflush($handle);
        flock($handle, LOCK_UN);
      }
      fclose($handle);
    }
  }

  private function save_const(&$content, $name, $value)
  {
    switch($value)
    {
      case '':
        $value = 'false';
        break;

      case 'on':
        $value = 'true';
        break;

      default:
        if(is_numeric($value)) $value = (int)$value;
        if(is_string($value)) $value = "'".d_e($value)."'";
        break;
    }


    $const = "define('$name',$value);\n";

    if(stripos($content, "define('$name'"))
    {
      $content = d_regex_replace('/define\(\''.$name.'\'.*?\n/sm', $const, $content);
    }
    else
    {
      $content .= "\n".$const;
    }

    $content = d_regex_replace('/\n+/s', "\n", $content);
  }

  public function save_all()
  {
    $_POST = array_map('stripslashes_deep', $_POST);

    if($handle = fopen($this->file, 'a+'))
    {
      if(flock($handle, LOCK_EX))
      {
        clearstatcache();
        $filesize = filesize($this->file);
        $content = $filesize > 0 ? fread($handle, $filesize) : '';

        foreach($this->consts as $name)
        {
          $this->save_const($content, $name, $_POST[$name]);
        }

        ftruncate($handle, 0);
        rewind($handle);
        fwrite($handle, $content);
        fflush($handle);
        flock($handle, LOCK_UN);

        $this->backup($content);
      }
      fclose($handle);
    }
  }



  private function backup($content = '', $file_name = '')
  {
    if(empty($file_name)) $file_name = $this->file_backup;

    if(empty($content) && $handle = fopen($this->file, 'a+'))
    {
      if(flock($handle, LOCK_EX))
      {
        clearstatcache();
        $filesize = filesize($this->file);
        $content = $filesize > 0 ? fread($handle, $filesize) : '<?php';
        flock($handle, LOCK_UN);
      }
      fclose($handle);
    }

    $content =  d_regex_replace('/define\(\'DBCACHE_.*?\n/s', '', $content);

    $content =  d_regex_replace('/\n\n+/s', "\n", $content);

    if($handle = fopen($file_name, 'a+'))
    {
      if(flock($handle, LOCK_EX))
      {
        ftruncate($handle, 0);
        rewind($handle);
        fwrite($handle, $content);
        fflush($handle);
        flock($handle, LOCK_UN);
      }
      fclose($handle);
    }
  }

  private function restore($file_name, $handle = null)
  {
    $content = $this->get_backup($file_name);

    if($content == null) return;

    if($handle != null)
    {
      ftruncate($handle, 0);
      rewind($handle);
      fwrite($handle, $content);
      fflush($handle);
    }
    else
    {
      if($handle = fopen($this->file, 'a+'))
      {
        if(flock($handle, LOCK_EX))
        {
          ftruncate($handle, 0);
          rewind($handle);
          fwrite($handle, $content);
          fflush($handle);
          flock($handle, LOCK_UN);
        }
        fclose($handle);
      }
    }
  }

  private function get_backup($file_name = '')
  {
    if(empty($file_name)) $file_name = $this->file_backup;

    if($handle = fopen($file_name, 'a+'))
    {
      if(flock($handle, LOCK_EX))
      {
        clearstatcache();
        $filesize = filesize($this->file);
        $content = $filesize > 0 ? fread($handle, $filesize) : '';
        flock($handle, LOCK_UN);
      }
      else
      {
        return null;
      }
      fclose($handle);
    }
    else
    {
      return null;
    }

    return $content;
  }


  private function check_config($content)
  {
    if(substr($content, 0, 5) != '<?php') return false;

    if(d_regex_is_match('/define.*?(?<!;)$/m', $content)) return false;

    return true;
  }

  private function check()
  {
    $status = '';

    $this->save('D_CHECK', date('Y.m.d h:i:s'));
    if($this->count_consts_regexp('/D_CHECK/s') <= 0)
    {
      $status .= 'Файл конфига не доступен для записи <br><b style="color:#777; font-size:11px; font-weight:normal;">['.$this->file.']</b><br>Установите права 0777 на папку с плагином.<br><br>';
    }
    else
    {
      $this->remove_consts_regexp('/define\(\'D_CHECK\'.*?\n/s');
    }

    return $status;
  }


  public function remove_consts_regexp($pattern)
  {
    if($handle = fopen($this->file, 'a+'))
    {
      if(flock($handle, LOCK_EX))
      {
        clearstatcache();
        $filesize = filesize($this->file);
        $content = $filesize > 0 ? fread($handle, $filesize) : '';

        $content =  d_regex_replace($pattern, '', $content);

        $content =  d_regex_replace('/\n\n+/s', "\n\n", $content);

        ftruncate($handle, 0);
        rewind($handle);
        fwrite($handle, $content);
        fflush($handle);
        flock($handle, LOCK_UN);
      }
      fclose($handle);
    }
  }

  public function count_consts_regexp($pattern)
  {
    $result = 0;

    if($handle = fopen($this->file, 'a+'))
    {
      if(flock($handle, LOCK_EX))
      {
        clearstatcache();
        $filesize = filesize($this->file);
        $content = $filesize > 0 ? fread($handle, $filesize) : '';

        $mathces = d_regex_all($pattern, $content);

        $result = count($mathces);

        flock($handle, LOCK_UN);
      }
      fclose($handle);
    }

    return $result;
  }



  public function add_options_page()
  {
    add_options_page('d-wp', 'd-wp', 'manage_options', 'd', array($this, 'html'));
    //add_menu_page('d-wp', 'd-wp', 'manage_options', 'd', array($this, 'html'), '', 2);
  }

  public function add_settings_link($links)
  {
    array_unshift($links, '<a href="'.admin_url('options-general.php').'?page=d">'.translate('Settings').'</a>');
    return $links;
  }


  public function html()
  {
    $d_config = d_config::get();

    $d_cache = d_cache::get();

    if(!empty($_POST['d_cmd']))
    {
      switch($_POST['d_cmd'])
      {
        case 'save_settings':
          $d_config->save_all();
          break;
        case 'cache_clear_all':
          $d_cache->clear_all();
          break;
        case 'cache_clear_posts':
          $d_cache->clear_posts();
          break;
        case 'cache_clear_index':
          $d_cache->clear_index();
          break;
        case 'cache_clear_db':
          $d_cache->clear_db();
          break;
      }

      echo '<script>location.reload()</script>';
      return;
    }


    $status = $this->check();
    $status .= $d_cache->check();


    $current_version = $d_config->version();

    /*$last_version = file_get_contents('http://d-wp.ru/versions/last_version');
    if(!d_regex_is_match('/\d+.\d+(.\d+|)/si', $last_version)) $last_version = null;*/


    $html = '';

    $html .= '<style>.notice{display:none}</style>';

    $html .= '<div class="wrap">';

    $html .= '<div style="position:relative; height:60px;">';
    $html .= '<h1 style="position:absolute">d-wp</h1>';
    $html .= '<div style="position:absolute; top:17px; left:70px;"><a href="http://d-wp.ru">Страница плагина</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href="http://d-wp.ru/help">Справочная информация</a></div>';
    $html .= '</div>';

    $html .= '<table class="wp-list-table widefat striped" style="width:370px">';
    $html .= '<thead>';
    $html .= '<tr>';
    $html .= '<td style="width:160px"><b>Плагин</b></td>';
    $html .= '<td></td>';
    $html .= '</tr>';
    $html .= '</thead>';
    $html .= '<tbody>';

    $html .= '<tr>';
    $html .= '<td><b>Состояние:</b></td>';
    $html .= '<td>'.(empty($status) ? '<b style="color:green">OK</b>' : '<b style="color:red">'.$status.'</b>').'</td>';
    $html .= '</tr>';

    $html .= '<tr>';
    $html .= '<td><b>Версия:</b></td>';
    $html .= '<td><b>'.$current_version.'</b></td>';
    $html .= '</tr>';


    $html .= '</tbody>';
    $html .= '</table>';

    $html .= '<br>';

    $html .= '<table class="wp-list-table widefat striped" style="width:370px">';
    $html .= '<thead>';
    $html .= '<tr>';
    $html .= '<td style="width:160px"><b>'.$d_config->html_remote_link('cache_clear_all', 'Кэш').'</b></td>';
    $html .= '<td style="width:60px"></td>';
    $html .= '<td><b>'.$d_config->html_button_d_cmd('cache_clear_all', 'Очистить').'</b></td>';
    $html .= '</tr>';
    $html .= '</thead>';
    $html .= '<tbody>';
    $html .= '<tr>';
    $html .= '<td><b>'.$d_config->html_remote_link('cache_clear_posts', 'Записи и страницы').'</b></td>';
    $html .= '<td><b>'.$d_cache->count_posts().'</b></td>';
    $html .= '<td>'.$d_config->html_button_d_cmd('cache_clear_posts', 'Очистить').'</td>';
    $html .= '</tr>';
    $html .= '<tr>';
    $html .= '<td><b>'.$d_config->html_remote_link('cache_clear_index', 'Индексные страницы').'</b></td>';
    $html .= '<td><b>'.$d_cache->count_index().'</b></td>';
    $html .= '<td>'.$d_config->html_button_d_cmd('cache_clear_index', 'Очистить').'</td>';
    $html .= '</tr>';
    $html .= '<tr>';
    $html .= '<td><b>'.$d_config->html_remote_link('cache_clear_db', 'Запросы к БД').'</b></td>';
    $html .= '<td><b>'.$d_cache->count_db().'</b></td>';
    $html .= '<td>'.$d_config->html_button_d_cmd('cache_clear_db', 'Очистить').'</td>';
    $html .= '</tr>';
    $html .= '</tbody>';
    $html .= '</table>';

    $html .= '<br><hr>';


    $html .= '<form action="options-general.php?page=d" method="post">';

    foreach($d_config->categories as $category) $html .= $category->html();

    $html .= '<input type="hidden" name="d_cmd" value="save_settings">';
    $html .= '<p><input type="submit" class="button-primary" value="Сохранить"></p>';
    $html .= '</form>';

    $html .= '</div>'; // wrap

    echo $html;
  }


  private function init()
  {
    $checkbox = 'checkbox';
    $input = 'input';
    $textarea = 'textarea';

    $this

      // Настройки плагина
      ->add(d_category::get('Плагин')
        ->add(d_subcategory::get()
          ->add(d_item::get('Удаленное управление', 'D_REMOTE_CONTROL')
            ->add(d_item::get('Пароль для удаленного управления', 'D_PASSWORD', $input))
          )
          ->add(d_item::get('Укажите URL страниц на которых плагин будет полностью отключен', 'D_GLOBAL_IGNORE')
            ->add(d_item::get('(* - любая последовательность)', 'D_GLOBAL_IGNORE_URLS', $textarea))
          )
          ->add(d_item::get('Код php, который необходимо выполнить до загрузки WordPress или кэша', 'D_PRE_PHP')
            ->add(d_item::get('include директория по умолчанию — корневая папка плагина [d]', 'D_PRE_PHP_CODE', $textarea))
          )
        )
      )

      // Кэш
      ->add(d_category::get('Кэш')
        ->add(d_subcategory::get()
          ->add(d_item::get('Не кэшировать страницы со следующими URL адресами', 'CACHE_IGNORE_URLS')
            ->add(d_item::get('(* - любая последовательность)', 'CACHE_IGNORE_URLS_LIST', $textarea))
          )
          ->add(d_item::get('Кэшировать записи и страницы', 'CACHE')
            ->add(d_item::get('Время жизни кэша в минутах (0 — не ограничено)', 'CACHE_LIFETIME', $input))
            ->add(d_item::get('Сбрасывать кэш всех записей при публикации, изменении или удалении других записей', 'CACHE_CLEAR_ALL_WHEN_SAVE_OR_DELETE_POSTS'))
          )
          ->add(d_item::get('Кэшировать индексные страницы', 'CACHE_INDEX')
            ->add(d_item::get('Время жизни кэша в минутах (0 — не ограничено)', 'CACHE_INDEX_LIFETIME', $input))
            ->add(d_item::get('Сбрасывать кэш индексных страниц при публикации, изменении или удалении записей', 'CACHE_INDEX_CLEAR_ALL_WHEN_SAVE_OR_DELETE_POSTS'))
            ->add(d_item::get('Кэшировать, даже если есть запланированные (если включаете, задайте время жизни кэша)', 'CACHE_INDEX_IF_FUTURE_POSTS_EXISTS'))
          )
          ->add(d_item::get('Кэшировать запросы к БД','CACHE_DB')
          )
        )
        ->add(d_subcategory::get('Уменьшение нагрузки на БД')
          ->add(d_item::get('Купировать запросы к wp_postmeta (включите, если не используете дополнительные поля)', 'DB_CROP_POSTMETA'))
          ->add(d_item::get('Купировать запросы к wp_comments и wp_commentmeta (включите, если не используете комментарии)', 'DB_CROP_COMMENTS'))
          ->add(d_item::get('Купировать запросы к wp_links (включите, если не используете ссылки на другие блоги, блогролл)', 'DB_CROP_LINKS'))
        )
      )

      // SEO
      ->add(d_category::get('SEO оптимизация')
        ->add(d_subcategory::get('Автоматическое формирование title и description')
          ->add(d_item::get('Перезаписывать заголовки (title)', 'SEO_TITLE')
            ->add(d_item::get('Добавлять к заголовкам имя сайта', 'SEO_TITLE_SITE_NAME'))
            ->add(d_item::get('Разделитель заголовка и имени сайта', 'SEO_TITLE_SITE_SEP', $input, 'width:50px; text-align:center;'))
          )
          ->add(d_item::get('Создавать описания для страниц (description)', 'SEO_DESCRIPTION'))
        )
        ->add(d_subcategory::get('Другие метатеги')
          ->add(d_item::get('Удалять ссылку на профиль метаданных', 'HIDE_LINK_PROFILE'))
          ->add(d_item::get('Удалять ссылку на Pingback (XML-RPC) интерфейс', 'HIDE_LINK_PINGBACK'))
          ->add(d_item::get('Удалять ссылку на RDS (XML-RPC) интерфейс', 'HIDE_LINK_XMLRPC'))
          ->add(d_item::get('Удалять ссылку на манифест Windows Live Writer', 'HIDE_LINK_WLW'))
          ->add(d_item::get('Удалять ссылку на короткий URL поста', 'HIDE_LINK_SHORT'))
          ->add(d_item::get('Удалять метатег generator с указанием версии WordPress', 'HIDE_TAG_WP_GENERATOR'))
          ->add(d_item::get('Удалить canonical ссылку', 'HIDE_LINK_CANONICAL'))
          ->add(d_item::get('Удалить ссылки на prev и next', 'HIDE_LINK_NEXT_PREV'))
          ->add(d_item::get('Добавлять ссылку на fav иконку', 'APPEND_LINK_FAVICON'))
        )
        ->add(d_subcategory::get('Индексация')
          ->add(d_item::get('noindex для страниц архивов', 'SEO_NOINDEX_DATE'))
          ->add(d_item::get('noindex для страниц поиска', 'SEO_NOINDEX_SEARCH'))
          ->add(d_item::get('noindex для страниц категорий', 'SEO_NOINDEX_CATEGORY'))
          ->add(d_item::get('noindex для страниц меток', 'SEO_NOINDEX_TAG'))
          ->add(d_item::get('noindex для страниц автора', 'SEO_NOINDEX_AUTHOR'))
          ->add(d_item::get('noindex для страниц навигации', 'SEO_NOINDEX_PAGED'))
          ->add(d_item::get('Перенаправлять (301) с несуществующих страниц (404) на случайный пост (перенаправление запоминается)', 'RND404'))
          ->add(d_item::get('Транслитерировать ссылки (аналог «rus to lat»)', 'TRANSLIT'))
        )
        ->add(d_subcategory::get('Дубли контента')
          ->add(d_item::get('Удалить архивы дат', 'REMOVE_ARCHIVES_DATES'))
          ->add(d_item::get('Удалить архивы тегов', 'REMOVE_ARCHIVES_TAGS'))
          ->add(d_item::get('Удалить архивы пользователей', 'REMOVE_ARCHIVES_AUTHORS'))
          ->add(d_item::get('Удалить пагинацию у постов', 'REMOVE_POSTS_PAGINATION'))
          ->add(d_item::get('Удалить страницы вложений', 'REMOVE_ATTACHMENT_PAGES'))
        )
        ->add(d_subcategory::get('Темы')
          ->add(d_item::get('Активировать первую найденную в папке themes тему', 'THEME_ACTIVATE_FIRST'))
          ->add(d_item::get('Удалять исходящие ссылки', 'REMOVE_EXTERNAL_LINKS'))
          ->add(d_item::get('Удалить стили recentcomments', 'REMOVE_RECENTCOMMENTS_STYLE'))
          ->add(d_item::get('Удалять комментарии кода из HTML', 'REMOVE_HTML_COMMENTS'))
          ->add(d_item::get('Сжимать HTML', 'MINIFY_HTML'))
        )
      )

      // Функционал
      ->add(d_category::get('Функционал сайта')
        ->add(d_subcategory::get()
          ->add(d_item::get('Публиковать запланированные посты', 'PUB_FUTURE_POSTS')
            ->add(d_item::get('Автоматически отключать функцию, если все посты опубликованы (уменьшает нагрузку на БД)', 'PUB_FUTURE_POSTS_AUTOOFF'))
          )
          ->add(d_item::get('Отключить комментарии', 'REMOVE_COMMENTS'))
          ->add(d_item::get('Отключить проверку форм (если не работают ajax запросы)', 'DISABLE_WP_VERIFY_NONCE'))
          ->add(d_item::get('Отключить RSS ленты', 'HIDE_LINK_FEEDS'))
          ->add(d_item::get('Отключить Emoji', 'REMOVE_EMOJIS'))
          ->add(d_item::get('Отключить автообновление', 'DISABLE_AUTOUPDATE'))
          ->add(d_item::get('Отключить крон', 'DISABLE_CRON'))
          ->add(d_item::get('Отключить ревизии постов', 'DISABLE_REVISIONS'))
          ->add(d_item::get('Отключить корзину', 'DISABLE_TRASH'))
          ->add(d_item::get('Не отображать панель администратора на сайте', 'HIDE_ADMIN_BAR'))
          ->add(d_item::get('Отображать панель отладки (включать только на время проверки работы плагина)', 'SHOW_DEBUG_BAR'))
        )
      )

      // Контент
      ->add(d_category::get('Контент')
        ->add(d_subcategory::get()

          ->add(d_item::get('Выполнять php в содержимом статьи (вставлять с &lt;?php...?&gt;, include директория по умолчанию — корневая папка плагина [d])', 'EXEC_PHP')
            ->add(d_item::get('Не кэшировать результат обработки php', 'EXEC_PHP_DO_NOT_CACHE'))
          )

          ->add(d_item::get('Вывести в &lt;head&gt;', 'D_INSERT_HEAD')
            ->add(d_item::get('Укажите текст, html, css, js, который необходимо вывести в теге &lt;head&gt;:', 'D_INSERT_HEAD_CONTENT', $textarea))
          )
          ->add(d_item::get('Вывести до содержимого статьи', 'D_INSERT_BEFORE')
            ->add(d_item::get('Укажите текст, html, css, js или php (если включен выше), который необходимо вывести до содержимого статьи:', 'D_INSERT_BEFORE_CONTENT', $textarea))
          )
          ->add(d_item::get('Вывести после содержимого статьи', 'D_INSERT_AFTER')
            ->add(d_item::get('Укажите текст, html, css, js или php (если включен выше), который необходимо вывести до содержимого статьи:', 'D_INSERT_AFTER_CONTENT', $textarea))
          )
          ->add(d_item::get('Вывести iframe поверх сайта', 'IFRAME')
            ->add(d_item::get('URL ([title] для передачи title страницы в URL, например, http://site.ru/?q=[title])<br>&nbsp;&nbsp;', 'IFRAME_URL', $input, 'width:400px'))
            ->add(d_item::get('Выводить iframe через ajax', 'IFRAME_AJAX'))
            ->add(d_item::get('Выводить белый блок div на весь экран, чтобы закрыть содержимое сайта на время загрузки фрейма', 'IFRAME_DIV', $checkbox))
          )
          ->add(d_item::get('Имитировать виждет «Поиск»', 'WIDGET_SEARCH')
            ->add(d_item::get('Заголовок', 'WIDGET_SEARCH_TITLE', $input))
          )
          ->add(d_item::get('Имитировать виждет «Меню»', 'WIDGET_MENU')
            ->add(d_item::get('Заголовок', 'WIDGET_MENU_TITLE', $input))
            ->add(d_item::get('Главная', 'WIDGET_MENU_MAIN'))
            ->add(d_item::get('Контакты', 'WIDGET_MENU_CONTACTS'))
            ->add(d_item::get('Категории', 'WIDGET_MENU_CATEGORIES'))
          )
          ->add(d_item::get('Имитировать виждет «Случайные посты»', 'WIDGET_RANDOM_POSTS')
            ->add(d_item::get('Заголовок', 'WIDGET_RANDOM_POSTS_TITLE', $input))
            ->add(d_item::get('Количество', 'WIDGET_RANDOM_POSTS_COUNT', $input))
          )
          ->add(d_item::get('Вывести счетчик статистики liveinternet.ru', 'WIDGET_STAT_LIVEINTERNET')
            ->add(d_item::get('Заголовок', 'WIDGET_STAT_LIVEINTERNET_TITLE', $input))
          )
        )
      )
    ;


    foreach($this->categories as $category)
    {
      foreach($category->subcategories as $subcategory)
      {
        foreach($subcategory->items as $item)
        {
          $this->consts[] = $item->name;

          foreach($item->subitems as $subitem)
          {
            $this->consts[] = $subitem->name;
          }
        }
      }
    }
  }



  private function html_remote_link($cmd, $anchor)
  {
    return (D_REMOTE_CONTROL && D_PASSWORD !== false) ? '<a href="/index.php?d_cmd='.$cmd.'&d_pass='.D_PASSWORD.'" target="_blank"">'.$anchor.'</a>' : $anchor;
  }

  private function html_button_d_cmd($d_cmd, $text)
  {
    return '<form action="options-general.php?page=d" method="post"><input type="submit" class="button" value="'.$text.'"><input type="hidden" name="d_cmd" value="'.$d_cmd.'"></form>';
  }


  private function version()
  {
    $d = get_plugins('/d');
    return $d['d.php']['Version'];
  }
}


class d_category
{
  public $title;

  public $subcategories = array();

  public function __construct($title)
  {
    $this->title = $title;
  }

  public static function get($title = '')
  {
    return new d_category($title);
  }

  public function add($subcategory)
  {
    $this->subcategories[] = $subcategory;
    return $this;
  }

  public function html()
  {
    $html = "<h3>$this->title</h3>";

    foreach($this->subcategories as $subcategory) $html .= $subcategory->html();

    $html .= '<hr>';

    return $html;
  }
}


class d_subcategory
{
  public $title;

  public $items = array();

  public function __construct($title)
  {
    $this->title = $title;
  }

  public static function get($title = '')
  {
    return new d_subcategory($title);
  }


  public function add($d_item)
  {
    $this->items[] = $d_item;
    return $this;
  }

  public function html()
  {
    $html = empty($this->title) ? '' : "<h4>$this->title</h4>";

    foreach($this->items as $item) $html .= $item->html();

    return $html;
  }
}


class d_item
{
  public $title;

  public $name;

  public $type;

  public $style;

  public $subitems = array();


  public function __construct($title, $name, $type, $style)
  {
    $this->title = $title;
    $this->name = $name;
    $this->type = $type;
    $this->style = $style;
  }

  public static function get($title, $name, $type = 'checkbox', $style = '')
  {
    return new d_item($title, $name, $type, $style);
  }

  public function add($d_item)
  {
    $this->subitems[] = $d_item;
    return $this;
  }


  public function html($sub = false)
  {
    $subitems_exists = count($this->subitems) > 0;

    $html = '';

    switch($this->type)
    {
      case 'checkbox':
        $js = $subitems_exists ? " onclick=\"document.getElementById('block_$this->name').style.display = document.getElementById('$this->name').checked ? 'block' : 'none';\"" : '';
        $html .= '<p>'.($sub ? '&nbsp;&nbsp;&nbsp;&nbsp;' : '').'<input '.$js.' type="checkbox" id="'.$this->name.'" name="'.$this->name.'"'.($this->d_constant() ? ' checked' : '').'><label for="'.$this->name.'">'.$this->title.'</label></p>';
        break;

      case 'input':
        $html .= '<p>'.($sub ? '&nbsp;&nbsp;&nbsp;&nbsp;' : '').'<label for="'.$this->name.'">'.$this->title.'</label>&nbsp;<input type="text" id="'.$this->name.'" name="'.$this->name.'" value="'.d_d($this->d_constant()).'" style="'.$this->style.'"></p>';
        break;

      case 'textarea':
        $html .= '<p>'.(empty($this->title) ? '' : ($sub ? '&nbsp;&nbsp;&nbsp;&nbsp;' : '').'<label for="'.$this->name.'">'.$this->title.'</label><br>').($sub ? '&nbsp;&nbsp;&nbsp;&nbsp;' : '').'<textarea id="'.$this->name.'" name="'.$this->name.'" wrap="off" style="'.(empty($this->style) ? 'width:700px; height:100px;' : $this->style).'">'.d_d($this->d_constant()).'</textarea></p>';
        break;
    }

    if($subitems_exists)
    {
      $html .= '<div id="block_'.$this->name.'" style="display:'.($this->d_constant() ? 'block' : 'none').'">';

      foreach($this->subitems as $subitem) $html .= $subitem->html(true);

      $html .= '</div>';
    }

    return $html;
  }


  private function d_constant()
  {
    if(defined($this->name)) return constant($this->name);
    else return '';
  }
}

d_config::get()->load();